From b9e3f46b6c5b3291b5f1a2537ae8d909810673ab Mon Sep 17 00:00:00 2001
From: Tomas Vondra <tomas.vondra@postgresql.org>
Date: Wed, 16 Aug 2023 16:05:42 +0200
Subject: [PATCH 4/4] update replorigin_session_origin_lsn

---
 src/backend/replication/logical/proto.c  |  4 ++--
 src/backend/replication/logical/worker.c | 11 +++++++++++
 src/include/replication/logicalproto.h   |  1 +
 3 files changed, 14 insertions(+), 2 deletions(-)

diff --git a/src/backend/replication/logical/proto.c b/src/backend/replication/logical/proto.c
index f11dd4600d6..76b34880de0 100644
--- a/src/backend/replication/logical/proto.c
+++ b/src/backend/replication/logical/proto.c
@@ -698,9 +698,9 @@ logicalrep_read_sequence(StringInfo in, LogicalRepSequence *seqdata)
 {
 	LogicalRepRelId relid;
 
-	/* XXX skipping flags and lsn */
+	/* XXX skipping flags */
 	pq_getmsgint(in, 1);
-	pq_getmsgint64(in);
+	seqdata->lsn = pq_getmsgint64(in);
 
 	/* read the relation id */
 	relid = pq_getmsgint(in, 4);
diff --git a/src/backend/replication/logical/worker.c b/src/backend/replication/logical/worker.c
index ae0a8b19a3b..ee05833143a 100644
--- a/src/backend/replication/logical/worker.c
+++ b/src/backend/replication/logical/worker.c
@@ -1515,7 +1515,18 @@ apply_handle_sequence(StringInfo s)
 	 * remote transaction, i.e. for non-transactional sequence updates.)
 	 */
 	if (!in_remote_transaction)
+	{
 		CommitTransactionCommand();
+
+		/*
+		 * Update origin state so we don't try applying this sequence
+		 * change in case of crash.
+		 *
+		 * XXX We don't have replorigin_session_origin_timestamp, but we
+		 * can just leave that set to 0.
+		 */
+		replorigin_session_origin_lsn = seq.lsn;
+	}
 }
 
 /*
diff --git a/src/include/replication/logicalproto.h b/src/include/replication/logicalproto.h
index cb30c46a165..f6bbb74d780 100644
--- a/src/include/replication/logicalproto.h
+++ b/src/include/replication/logicalproto.h
@@ -128,6 +128,7 @@ typedef struct LogicalRepTyp
 /* Sequence info */
 typedef struct LogicalRepSequence
 {
+	XLogRecPtr	lsn;			/* LSN of the sequence change */
 	Oid			remoteid;		/* unique id of the remote sequence */
 	char	   *nspname;		/* schema name of remote sequence */
 	char	   *seqname;		/* name of the remote sequence */
-- 
2.41.0

